#ifndef _BUMP_ALLOC
#define _BUMP_ALLOC

#include "Types.h"

typedef void* BumpArena;

BumpArena B_create(usize size);
Result    B_destroy(BumpArena b);

void*  B_alloc(BumpArena b, usize size);
int    B_resizeable(BumpArena b, void* ptr);
Result B_resize(BumpArena b, usize size);
Result B_free(BumpArena b);

#endif /* _BUMP_ALLOC */
